@@ -16,8 +16,9 @@ from rasterio.rio.overview import get_maximum_overview_level
 from rasterio.vrt import WarpedVRT
 from rasterio.warp import calculate_default_transform, transform_bounds
 
-from .. import constants, reader
-from ..errors import ExpressionMixingWarning, TileOutsideBounds
+from .. import reader
+from ..constants import WEB_MERCATOR_TMS, WGS84_CRS, BBox, Indexes, NoData
+from ..errors import ExpressionMixingWarning, NoOverviewWarning, TileOutsideBounds
 from ..expression import apply_expression, parse_expression
 from ..models import ImageData, ImageStatistics, Info
 from ..utils import create_cutline, has_alpha_band, has_mask_band
@@ -64,13 +65,13 @@ class COGReader(BaseReader):
     dataset: Union[DatasetReader, DatasetWriter, MemoryFile, WarpedVRT] = attr.ib(
         default=None
     )
-    tms: TileMatrixSet = attr.ib(default=constants.WEB_MERCATOR_TMS)
+    tms: TileMatrixSet = attr.ib(default=WEB_MERCATOR_TMS)
     minzoom: int = attr.ib(default=None)
     maxzoom: int = attr.ib(default=None)
     colormap: Dict = attr.ib(default=None)
 
     # Define global options to be forwarded to functions reading the data (e.g `rio_tiler.reader.read`)
-    nodata: Optional[Union[float, int, str]] = attr.ib(default=None)
+    nodata: Optional[NoData] = attr.ib(default=None)
     unscale: Optional[bool] = attr.ib(default=None)
     resampling_method: Optional[Resampling] = attr.ib(default=None)
     vrt_options: Optional[Dict] = attr.ib(default=None)
@@ -100,7 +101,7 @@ class COGReader(BaseReader):
         self.nodata = self.nodata if self.nodata is not None else self.dataset.nodata
 
         self.bounds = transform_bounds(
-            self.dataset.crs, constants.WGS84_CRS, *self.dataset.bounds, densify_pts=21
+            self.dataset.crs, WGS84_CRS, *self.dataset.bounds, densify_pts=21
         )
         if self.minzoom is None or self.maxzoom is None:
             self._set_zooms()
@@ -108,6 +109,14 @@ class COGReader(BaseReader):
         if self.colormap is None:
             self._get_colormap()
 
+        if min(
+            self.dataset.width, self.dataset.height
+        ) > 512 and not self.dataset.overviews(1):
+            warnings.warn(
+                "The dataset has no Overviews. rio-tiler performances might be impacted.",
+                NoOverviewWarning,
+            )
+
     def close(self):
         """Close rasterio dataset."""
         if self.filepath:
@@ -163,11 +172,6 @@ class COGReader(BaseReader):
             """Return band description."""
             return self.dataset.descriptions[ix - 1] or ""
 
-        indexes = self.dataset.indexes
-        band_descr = [(f"{ix}", _get_descr(ix)) for ix in indexes]
-        band_meta = [(f"{ix}", self.dataset.tags(ix)) for ix in indexes]
-        colorinterp = [self.dataset.colorinterp[ix - 1].name for ix in indexes]
-
         if has_alpha_band(self.dataset):
             nodata_type = "Alpha"
         elif has_mask_band(self.dataset):
@@ -177,27 +181,40 @@ class COGReader(BaseReader):
         else:
             nodata_type = "None"
 
-        other_meta = {}
-        if self.dataset.scales[0] and self.dataset.offsets[0]:
-            other_meta.update(
-                {"scale": self.dataset.scales[0], "offset": self.dataset.offsets[0]}
-            )
-
-        if self.colormap:
-            other_meta.update({"colormap": self.colormap})
-
         meta = {
             "bounds": self.bounds,
             "center": self.center,
             "minzoom": self.minzoom,
             "maxzoom": self.maxzoom,
-            "band_metadata": band_meta,
-            "band_descriptions": band_descr,
+            "band_metadata": [
+                (f"{ix}", self.dataset.tags(ix)) for ix in self.dataset.indexes
+            ],
+            "band_descriptions": [
+                (f"{ix}", _get_descr(ix)) for ix in self.dataset.indexes
+            ],
             "dtype": self.dataset.meta["dtype"],
-            "colorinterp": colorinterp,
+            "colorinterp": [
+                self.dataset.colorinterp[ix - 1].name for ix in self.dataset.indexes
+            ],
             "nodata_type": nodata_type,
+            # additional info (not in default model)
+            "driver": self.dataset.driver,
+            "count": self.dataset.count,
+            "width": self.dataset.width,
+            "height": self.dataset.height,
+            "overviews": self.dataset.overviews(1),
         }
-        meta.update(**other_meta)
+        if self.dataset.scales[0] and self.dataset.offsets[0]:
+            meta.update(
+                {"scale": self.dataset.scales[0], "offset": self.dataset.offsets[0]}
+            )
+
+        if self.colormap:
+            meta.update({"colormap": self.colormap})
+
+        if nodata_type == "Nodata":
+            meta.update({"nodata_value": self.nodata})
+
         return Info(**meta)
 
     def stats(
@@ -239,8 +256,9 @@ class COGReader(BaseReader):
         tile_y: int,
         tile_z: int,
         tilesize: int = 256,
-        indexes: Optional[Union[int, Sequence]] = None,
+        indexes: Optional[Indexes] = None,
         expression: Optional[str] = None,
+        tile_buffer: Optional[int] = None,
         **kwargs: Any,
     ) -> ImageData:
         """Read a Web Map tile from a COG.
@@ -278,6 +296,17 @@ class COGReader(BaseReader):
             indexes = parse_expression(expression)
 
         tile_bounds = self.tms.xy_bounds(*Tile(x=tile_x, y=tile_y, z=tile_z))
+        if tile_buffer:
+            x_res = (tile_bounds[2] - tile_bounds[0]) / tilesize
+            y_res = (tile_bounds[3] - tile_bounds[1]) / tilesize
+            tile_bounds = (
+                tile_bounds[0] - x_res * tile_buffer,
+                tile_bounds[1] - y_res * tile_buffer,
+                tile_bounds[2] + x_res * tile_buffer,
+                tile_bounds[3] + y_res * tile_buffer,
+            )
+            tilesize += tile_buffer * 2
+
         tile, mask = reader.part(
             self.dataset,
             tile_bounds,
@@ -298,9 +327,9 @@ class COGReader(BaseReader):
 
     def part(
         self,
-        bbox: Tuple[float, float, float, float],
+        bbox: BBox,
         dst_crs: Optional[CRS] = None,
-        bounds_crs: CRS = constants.WGS84_CRS,
+        bounds_crs: CRS = WGS84_CRS,
         max_size: int = 1024,
         indexes: Optional[Union[int, Sequence]] = None,
         expression: Optional[str] = None,
@@ -364,7 +393,7 @@ class COGReader(BaseReader):
 
     def preview(
         self,
-        indexes: Optional[Union[int, Sequence]] = None,
+        indexes: Optional[Indexes] = None,
         expression: Optional[str] = None,
         **kwargs: Any,
     ) -> ImageData:
@@ -412,7 +441,7 @@ class COGReader(BaseReader):
         self,
         lon: float,
         lat: float,
-        indexes: Optional[Union[int, Sequence]] = None,
+        indexes: Optional[Indexes] = None,
         expression: Optional[str] = None,
         **kwargs: Any,
     ) -> List:
@@ -456,9 +485,9 @@ class COGReader(BaseReader):
         self,
         shape: Dict,
         dst_crs: Optional[CRS] = None,
-        shape_crs: CRS = constants.WGS84_CRS,
+        shape_crs: CRS = WGS84_CRS,
         max_size: int = 1024,
-        indexes: Optional[Union[int, Sequence]] = None,
+        indexes: Optional[Indexes] = None,
         expression: Optional[str] = None,
         **kwargs: Any,
     ) -> ImageData:
@@ -526,6 +555,52 @@ class COGReader(BaseReader):
             data, mask, bounds=bounds, crs=dst_crs, assets=[self.filepath],
         )
 
+    def read(
+        self,
+        indexes: Optional[Indexes] = None,
+        expression: Optional[str] = None,
+        **kwargs: Any,
+    ) -> ImageData:
+        """Read the COG.
+
+        Args:
+            indexes (sequence of int or int, optional): Band indexes.
+            expression (str, optional): rio-tiler expression (e.g. b1/b2+b3).
+            kwargs (optional): Options to forward to the `rio_tiler.reader.read` function.
+
+        Returns:
+            rio_tiler.models.ImageData: ImageData instance with data, mask and input spatial info.
+
+        """
+        kwargs = {**self._kwargs, **kwargs}
+
+        if isinstance(indexes, int):
+            indexes = (indexes,)
+
+        if indexes and expression:
+            warnings.warn(
+                "Both expression and indexes passed; expression will overwrite indexes parameter.",
+                ExpressionMixingWarning,
+            )
+
+        if expression:
+            indexes = parse_expression(expression)
+
+        data, mask = reader.read(self.dataset, indexes=indexes, **kwargs)
+
+        if expression:
+            blocks = expression.lower().split(",")
+            bands = [f"b{bidx}" for bidx in indexes]
+            data = apply_expression(blocks, bands, data)
+
+        return ImageData(
+            data,
+            mask,
+            bounds=self.dataset.bounds,
+            crs=self.dataset.crs,
+            assets=[self.filepath],
+        )
+
 
 @attr.s
 class GCPCOGReader(COGReader):
@@ -561,13 +636,13 @@ class GCPCOGReader(COGReader):
     src_dataset: Union[DatasetReader, DatasetWriter, MemoryFile, WarpedVRT] = attr.ib(
         default=None
     )
-    tms: TileMatrixSet = attr.ib(default=constants.WEB_MERCATOR_TMS)
+    tms: TileMatrixSet = attr.ib(default=WEB_MERCATOR_TMS)
     minzoom: int = attr.ib(default=None)
     maxzoom: int = attr.ib(default=None)
     colormap: Dict = attr.ib(default=None)
 
     # Define global options to be forwarded to functions reading the data (e.g `rio_tiler.reader.read`)
-    nodata: Optional[Union[float, int, str]] = attr.ib(default=None)
+    nodata: Optional[NoData] = attr.ib(default=None)
     unscale: Optional[bool] = attr.ib(default=None)
     resampling_method: Optional[Resampling] = attr.ib(default=None)
     vrt_options: Optional[Dict] = attr.ib(default=None)
